@@ -45,6 +45,7 @@ extern "C" {
 #endif
 
 #include <stddef.h>
+#include <signal.h>
 #include <sys/types.h>
 
 typedef struct eio_req    eio_req;
@@ -57,7 +58,28 @@ typedef int (*eio_cb)(eio_req *req);
 #endif
 
 #ifndef EIO_STRUCT_STAT
-# define EIO_STRUCT_STAT struct stat
+# ifdef _WIN32
+#  define EIO_STRUCT_STAT struct _stati64
+#  define EIO_STRUCT_STATI64
+# else
+#  define EIO_STRUCT_STAT struct stat
+# endif
+#endif
+
+#ifdef _WIN32
+  typedef int      eio_uid_t;
+  typedef int      eio_gid_t;
+  typedef intptr_t eio_ssize_t; /* or SSIZE_T */
+  #if __GNUC__
+    typedef long long eio_ino_t;
+  #else
+    typedef __int64   eio_ino_t; /* unsigned not supported by msvc */
+  #endif
+#else
+  typedef uid_t    eio_uid_t;
+  typedef gid_t    eio_gid_t;
+  typedef ssize_t  eio_ssize_t;
+  typedef ino_t    eio_ino_t;
 #endif
 
 #ifndef EIO_STRUCT_STATVFS
@@ -106,7 +128,7 @@ struct eio_dirent
   unsigned short namelen; /* size of filename without trailing 0 */
   unsigned char type; /* one of EIO_DT_* */
   signed char score; /* internal use */
-  ino_t inode; /* the inode number, if available, otherwise unspecified */
+  eio_ino_t inode; /* the inode number, if available, otherwise unspecified */
 };
 
 /* eio_msync flags */
@@ -118,14 +140,12 @@ enum
 };
 
 /* eio_mtouch flags */
-
 enum
 {
   EIO_MT_MODIFY     = 1
 };
 
 /* eio_sync_file_range flags */
-
 enum
 {
   EIO_SYNC_FILE_RANGE_WAIT_BEFORE = 1,
@@ -133,10 +153,16 @@ enum
   EIO_SYNC_FILE_RANGE_WAIT_AFTER  = 4
 };
 
-typedef double eio_tstamp; /* feel free to use double in your code directly */
+/* eio_fallocate flags */
+enum
+{
+  EIO_FALLOC_FL_KEEP_SIZE = 1 /* MUST match the value in linux/falloc.h */
+};
+
+/* timestamps and differences - feel free to use double in your code directly */
+typedef double eio_tstamp;
 
 /* the eio request structure */
-
 enum
 {
   EIO_CUSTOM,
@@ -150,11 +176,11 @@ enum
   EIO_CHMOD, EIO_FCHMOD,
   EIO_CHOWN, EIO_FCHOWN,
   EIO_SYNC, EIO_FSYNC, EIO_FDATASYNC,
-  EIO_MSYNC, EIO_MTOUCH, EIO_SYNC_FILE_RANGE,
+  EIO_MSYNC, EIO_MTOUCH, EIO_SYNC_FILE_RANGE, EIO_FALLOCATE,
   EIO_MLOCK, EIO_MLOCKALL,
   EIO_UNLINK, EIO_RMDIR, EIO_MKDIR, EIO_RENAME,
   EIO_MKNOD, EIO_READDIR,
-  EIO_LINK, EIO_SYMLINK, EIO_READLINK,
+  EIO_LINK, EIO_SYMLINK, EIO_READLINK, EIO_REALPATH,
   EIO_GROUP, EIO_NOP,
   EIO_BUSY
 };
@@ -181,9 +207,9 @@ struct eio_req
 {
   eio_req volatile *next; /* private ETP */
 
-  ssize_t result;  /* result of syscall, e.g. result = read (... */
-  off_t offs;      /* read, write, truncate, readahead, sync_file_range: file offset, mknod: dev_t */
-  size_t size;     /* read, write, readahead, sendfile, msync, mlock, sync_file_range: length */
+  eio_ssize_t result;  /* result of syscall, e.g. result = read (... */
+  off_t offs;      /* read, write, truncate, readahead, sync_file_range, fallocate: file offset, mknod: dev_t */
+  size_t size;     /* read, write, readahead, sendfile, msync, mlock, sync_file_range, fallocate: length */
   void *ptr1;      /* all applicable requests: pathname, old name; readdir: optional eio_dirents */
   void *ptr2;      /* all applicable requests: new name or memory buffer; readdir: name strings */
   eio_tstamp nv1;  /* utime, futime: atime; busy: sleep time */
@@ -191,10 +217,16 @@ struct eio_req
 
   int type;        /* EIO_xxx constant ETP */
   int int1;        /* all applicable requests: file descriptor; sendfile: output fd; open, msync, mlockall, readdir: flags */
-  long int2;       /* chown, fchown: uid; sendfile: input fd; open, chmod, mkdir, mknod: file mode, sync_file_range: flags */
+  long int2;       /* chown, fchown: uid; sendfile: input fd; open, chmod, mkdir, mknod: file mode, sync_file_range, fallocate: flags */
   long int3;       /* chown, fchown: gid */
   int errorno;     /* errno value on syscall return */
 
+#if __i386 || __amd64
+  unsigned char cancelled;
+#else
+  sig_atomic_t cancelled;
+#endif
+
   unsigned char flags; /* private */
   signed char pri;     /* the priority */
 
@@ -210,10 +242,9 @@ struct eio_req
 
 /* _private_ request flags */
 enum {
-  EIO_FLAG_CANCELLED = 0x01, /* request was cancelled */
-  EIO_FLAG_PTR1_FREE = 0x02, /* need to free(ptr1) */
-  EIO_FLAG_PTR2_FREE = 0x04, /* need to free(ptr2) */
-  EIO_FLAG_GROUPADD  = 0x08  /* some request was added to the group */
+  EIO_FLAG_PTR1_FREE = 0x01, /* need to free(ptr1) */
+  EIO_FLAG_PTR2_FREE = 0x02, /* need to free(ptr2) */
+  EIO_FLAG_GROUPADD  = 0x04  /* some request was added to the group */
 };
 
 /* undocumented/unsupported/private helper */
@@ -245,7 +276,7 @@ void eio_set_idle_timeout (unsigned int seconds);
 
 unsigned int eio_nreqs    (void); /* number of requests in-flight */
 unsigned int eio_nready   (void); /* number of not-yet handled requests */
-unsigned int eio_npending (void); /* numbe rof finished but unhandled requests */
+unsigned int eio_npending (void); /* number of finished but unhandled requests */
 unsigned int eio_nthreads (void); /* number of worker threads in use currently */
 
 /*****************************************************************************/
@@ -262,6 +293,7 @@ eio_req *eio_mtouch    (void *addr, size_t length, int flags, int pri, eio_cb cb
 eio_req *eio_mlock     (void *addr, size_t length, int pri, eio_cb cb, void *data);
 eio_req *eio_mlockall  (int flags, int pri, eio_cb cb, void *data);
 eio_req *eio_sync_file_range (int fd, off_t offset, size_t nbytes, unsigned int flags, int pri, eio_cb cb, void *data);
+eio_req *eio_fallocate (int fd, int mode, off_t offset, size_t len, int pri, eio_cb cb, void *data);
 eio_req *eio_close     (int fd, int pri, eio_cb cb, void *data);
 eio_req *eio_readahead (int fd, off_t offset, size_t length, int pri, eio_cb cb, void *data);
 eio_req *eio_read      (int fd, void *buf, size_t length, off_t offset, int pri, eio_cb cb, void *data);
@@ -284,6 +316,7 @@ eio_req *eio_readdir   (const char *path, int flags, int pri, eio_cb cb, void *d
 eio_req *eio_rmdir     (const char *path, int pri, eio_cb cb, void *data);
 eio_req *eio_unlink    (const char *path, int pri, eio_cb cb, void *data);
 eio_req *eio_readlink  (const char *path, int pri, eio_cb cb, void *data); /* result=ptr2 allocated dynamically */
+eio_req *eio_realpath  (const char *path, int pri, eio_cb cb, void *data); /* result=ptr2 allocated dynamically */
 eio_req *eio_stat      (const char *path, int pri, eio_cb cb, void *data); /* stat buffer=ptr2 allocated dynamically */
 eio_req *eio_lstat     (const char *path, int pri, eio_cb cb, void *data); /* stat buffer=ptr2 allocated dynamically */
 eio_req *eio_statvfs   (const char *path, int pri, eio_cb cb, void *data); /* stat buffer=ptr2 allocated dynamically */
@@ -307,7 +340,7 @@ void eio_grp_cancel    (eio_req *grp); /* cancels all sub requests but not the g
 /* request api */
 
 /* true if the request was cancelled, useful in the invoke callback */
-#define EIO_CANCELLED(req)   ((req)->flags & EIO_FLAG_CANCELLED)
+#define EIO_CANCELLED(req)   ((req)->cancelled)
 
 #define EIO_RESULT(req)      ((req)->result)
 /* returns a pointer to the result buffer allocated by eio */
@@ -320,13 +353,11 @@ void eio_grp_cancel    (eio_req *grp); /* cancels all sub requests but not the g
 void eio_submit (eio_req *req);
 /* cancel a request as soon fast as possible, if possible */
 void eio_cancel (eio_req *req);
-/* destroy a request that has never been submitted */
-void eio_destroy (eio_req *req);
 
 /*****************************************************************************/
-/* convinience functions */
+/* convenience functions */
 
-ssize_t eio_sendfile_sync (int ofd, int ifd, off_t offset, size_t count);
+eio_ssize_t eio_sendfile_sync (int ofd, int ifd, off_t offset, size_t count);
 
 #ifdef __cplusplus
 }
